/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include <math.h>

#include "option.h"
#include "revision.h"
#include "registry.h"
#include "TinyCadView.h"
#include "TinyCad.h"
#include "MainFrm.h"
#include "DlgGetFindBox.h"


CDlgGetFindBox::CDlgGetFindBox() 
: CInitDialogBar()
{
	//{{AFX_DATA_INIT(CDlgGetFindBox)
	m_search_string = _T("");
	m_filter = 0;
	//}}AFX_DATA_INIT
	m_Symbol = NULL;
}

void CDlgGetFindBox::DoDataExchange(CDataExchange* pDX)
{
	CInitDialogBar::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgGetFindBox)
	DDX_Control(pDX, IDC_SHOW_SYMBOL, m_Show_Symbol);
	DDX_Control(pDX, FINDGET_LIST, m_List);
	DDX_Text(pDX, IDC_SEARCH_STRING, m_search_string);
	DDX_Radio(pDX, IDC_RADIO1, m_filter);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CDlgGetFindBox, CInitDialogBar)
	//{{AFX_MSG_MAP(CDlgGetFindBox)
	ON_LBN_SELCHANGE(FINDGET_LIST , OnListSelect )
	ON_EN_CHANGE(IDC_SEARCH_STRING, OnChangeSearchString)
	ON_WM_DRAWITEM()
	ON_LBN_DBLCLK(FINDGET_LIST, OnDblclkList)
	ON_BN_CLICKED(IDC_RADIO1, OnRadio1)
	ON_BN_CLICKED(IDC_RADIO2, OnRadio2)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


BOOL CDlgGetFindBox::OnInitDialogBar()
{
	CInitDialogBar::OnInitDialogBar();

	BuildSearchList();
	if (!m_Resize.m_hWnd)
	{
		m_Resize.Create( NULL, "", WS_VISIBLE|WS_CHILD,CRect(0,0,10,10), this, ID_RESIZE );
	}
	m_sizeUndockedDefault = m_sizeDefault;

	return TRUE;
}


// End the dialog when an item is clicked on
void CDlgGetFindBox::OnListSelect()
{
	CListBox *theListBox = (CListBox *)GetDlgItem(FINDGET_LIST);
	int WhichItem;

 	if ((WhichItem = theListBox->GetCurSel())!=LB_ERR) 
	{
		m_Symbol = static_cast<CLibraryStoreSymbol *>(theListBox->GetItemDataPtr( WhichItem ));
		GetDlgItem( IDC_SHOW_SYMBOL )->RedrawWindow();
	}
}




void CDlgGetFindBox::OnChangeSearchString() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CInitDialogBar::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.
	
	UpdateData( TRUE );
	BuildSearchList();
}

void CDlgGetFindBox::BuildSearchList()
{
	// Build the list box
	CListBox *theListBox = (CListBox *)GetDlgItem(FINDGET_LIST);
	theListBox->ResetContent();

	// Ensure the string is in Lower case for searching
	m_search_string.MakeLower();


	switch (m_filter)
	{
	case 0:
		{
			libraryIterator lib_it = static_cast<CTinyCadApp*>(AfxGetApp())->GetLibraryBegin();
			while (lib_it != static_cast<CTinyCadApp*>(AfxGetApp())->GetLibraryEnd()) 
			{
				CLibraryStore *aLibrary = *lib_it;
				aLibrary->Find(m_search_string,theListBox);

				++ lib_it;
			}
		}
		break;
	case 1:
		{
			MRUCollection::iterator it = m_most_recently_used.begin();
			while (it != m_most_recently_used.end())
			{
				CLibraryStoreSymbol* symbol = *it;
				BOOL ok = FALSE;

				libraryIterator lib_it = static_cast<CTinyCadApp*>(AfxGetApp())->GetLibraryBegin();
				while (!ok && lib_it != static_cast<CTinyCadApp*>(AfxGetApp())->GetLibraryEnd()) 
				{
					CLibraryStore *aLibrary = *lib_it;
					ok |= aLibrary->DoesExist( symbol );

					++ lib_it;
				}

				if (ok)
				{
					CString test = symbol->name + symbol->description;
					test.MakeLower();
					if (test.Find( m_search_string ) != -1)
					{
						int index = theListBox->AddString( symbol->name + " - " + symbol->description);
						theListBox->SetItemDataPtr( index, symbol );
					}
				}
				++ it;
			}
		}
		break;
	}

	// Try and find the selected symbol in the list
	for (int i = 0; i < theListBox->GetCount(); i++)
	{
		if (theListBox->GetItemDataPtr( i ) == m_Symbol)
		{
			theListBox->SetCurSel( i );
			break;
		}
	}
	if (i == theListBox->GetCount())
	{
		m_Symbol = NULL;
		GetDlgItem( IDC_SHOW_SYMBOL )->RedrawWindow();
	}
}

void CDlgGetFindBox::OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	CDC dc;
	dc.Attach( lpDrawItemStruct->hDC );


	dc.SelectStockObject( WHITE_BRUSH );
	dc.Rectangle( &lpDrawItemStruct->rcItem );

	if (m_Symbol != NULL)
	{
		Transform transform;
		Context q( &dc, transform );
		
		// Now access the symbol and draw it next to the name
		CPoint p;
		CTinyCadDoc doc;
		CDesignFileSymbol *pSymbol = m_Symbol->GetDesignSymbol( &doc );

		// Determine the rotation
		int orientation = m_Symbol->orientation;


		drawingCollection method;
		if (pSymbol)
		{
			pSymbol->GetMethod( p, method );
		}
		else
		{
			p = CPoint(-75,-15);
		}

		switch (orientation)
		{
		case 2:
			{
				CPoint q = p;
				p.x = q.y;
				p.y = q.x;
			}
			break;
		case 3:
			{
				CPoint q = p;
				p.x = q.y;
				p.y = q.x;
			}
		}
		int height = (lpDrawItemStruct->rcItem.bottom - lpDrawItemStruct->rcItem.top);
		int width = (lpDrawItemStruct->rcItem.right - lpDrawItemStruct->rcItem.left);

		// Determine the zoom factor
		int zoom = min( (width * 75) / fabs(p.x), (height*75)/fabs(p.y) );
		zoom = min( zoom, 100 );
		q.theTransform.SetZoom( zoom );

		height = (height * 100) / zoom;
		width = (width * 100) / zoom;
		// p.x = (p.x * zoom) / 100;
		// p.y = (p.y * zoom) / 100;


		// Draw the name
		q.SetTextAlign(TA_LEFT | TA_BOTTOM | TA_NOUPDATECP);
		q.SetBkMode( TRANSPARENT );

		// Now display the symbol
		CPoint old;

		switch (orientation)
		{
		case 2:
			old = q.theTransform.SetTRM( CPoint((width-p.x)/2, (height - p.y)/2) , CPoint(0,0), orientation );
			break;
		case 3:
			old = q.theTransform.SetTRM( CPoint(width / 2 + p.x/2, (height-p.y)/2) , CPoint(0,0), orientation );
			break;
		default:
			old = q.theTransform.SetTRM( CPoint((width-p.x)/2, (height-p.y)/2) , CPoint(0,0), orientation );
			break;
		}
		if (pSymbol)
		{
			drawingIterator paint_it = method.begin();
			while (paint_it != method.end())
			{
				(*paint_it)->Paint( q, draw_normal );
				++ paint_it;
			}

			delete pSymbol;
		}

		q.theTransform.EndTRM( old );
	}


	dc.Detach();

	// CInitDialogBar::OnDrawItem(nIDCtl, lpDrawItemStruct);
}

void CDlgGetFindBox::OnDblclkList() 
{
	AfxGetMainWnd()->PostMessage(WM_COMMAND, IDM_TOOLGET );
}

void CDlgGetFindBox::OnRadio1() 
{
	UpdateData( TRUE );
	BuildSearchList();
}

void CDlgGetFindBox::OnRadio2() 
{
	UpdateData( TRUE );
	BuildSearchList();
}



void CDlgGetFindBox::AddToMRU()
{
	// Is this symbol already in the MRU list?
	MRUCollection::iterator it = m_most_recently_used.begin();
	while (it != m_most_recently_used.end())
	{
		CLibraryStoreSymbol* symbol = *it;
		if (symbol == m_Symbol)
		{
			// Already in the MRU list
			return;
		}
		++ it;
	}

	m_most_recently_used.push_front( m_Symbol );

	// We keep only up to 20 symbols in this list
	if (m_most_recently_used.size() > 20)
	{
		m_most_recently_used.pop_back();
	}
}

void CDlgGetFindBox::OnSize(UINT nType, int cx, int cy) 
{
	CInitDialogBar::OnSize(nType, cx, cy);
	
	// Resize the list to the correct width
	if (m_List.m_hWnd)
	{
		CRect list_rect;
		m_List.GetWindowRect(list_rect);
		ScreenToClient( list_rect );

		int border_y = list_rect.left;
		int border_x = IsFloating() ? list_rect.left : list_rect.left + 4;
		list_rect.right = cx - border_x;
		list_rect.bottom = cy - border_y;
		m_List.MoveWindow( list_rect );

		CRect show_rect;
		m_Show_Symbol.GetWindowRect( show_rect );
		ScreenToClient( show_rect );
		show_rect.right = cx - border_x;
		m_Show_Symbol.MoveWindow( show_rect );
		
		CRect resize_rect;
		int width = 6;
		if (IsFloating())
		{
			resize_rect.left = -5;
			resize_rect.right = -3;
			resize_rect.top = 0;
			resize_rect.bottom = 1;
		}
		else
		{
			resize_rect.left = cx-width;
			resize_rect.right = cx;
			resize_rect.top = border_y;
			resize_rect.bottom = cy-2;
		}
		m_Resize.MoveWindow( resize_rect );
	}
}


CSize CDlgGetFindBox::CalcDynamicLayout( int nLength, DWORD dwMode )
{
	if (m_Resize.m_adjust_width != 0)
	{
		m_sizeDefault.cx += m_Resize.m_adjust_width;
		m_Resize.m_adjust_width = 0;
	}

	if (   (dwMode & LM_STRETCH)!=0
		|| (dwMode & LM_MRUWIDTH)!=0
		|| (dwMode & LM_COMMIT)!=0
		|| (dwMode & LM_HORZDOCK)!=0
		|| (dwMode & LM_VERTDOCK)!=0)
	{
	}
	else
	{
		// Otherwise store this length
		if ((dwMode & LM_LENGTHY)!=0)
		{
			m_sizeUndockedDefault.cy = nLength;
		}
		else
		{
			m_sizeUndockedDefault.cx = nLength;
		}
		m_sizeDefault = m_sizeUndockedDefault;
	}

	return CalcFixedLayout(dwMode & LM_STRETCH, dwMode & LM_HORZDOCK);
}

CSize CDlgGetFindBox::CalcFixedLayout( BOOL bStretch, BOOL bHorz )
{
	if (!IsFloating())
	{
		CWnd* pWnd=GetParent();
		CRect r;
		pWnd->GetClientRect(r);
		m_sizeDefault.cy = r.Height();
	}


	return IsFloating() ? m_sizeUndockedDefault : m_sizeDefault;
}



