/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/




// This handles the actual drawing of objects

#include "stdafx.h"
#include "TinyCadView.h"
#include "diag.h"
#include "colour.h"
#include "option.h"


// The editing dialogs
extern CDlgERCListBox theERCListBox;


/////// The Object classes /////////




// The Error Class //

#define ERRORSIZE	6

CDrawError::CDrawError(CTinyCadDoc *pDesign, CPoint NewA,int q)
: CDrawingObject( pDesign )
{
	a = NewA;
	b = NewA;
	m_segment=0;
	ErrorNumber = q;
	m_Undo_Level=0;
	m_Delete_Level=-1;
}


ObjType CDrawError::GetType()
{
  return xError;
}

CString CDrawError::GetName() const
{
  return "Error";
}

BOOL CDrawError::IsInside(int left,int right,int top,int bottom)
{
  return !( (a.x-ERRORSIZE<left && a.x+ERRORSIZE<=left) || (a.x-ERRORSIZE>right && a.x+ERRORSIZE>=right)
        || (a.y-ERRORSIZE<top && a.y+ERRORSIZE<=top) || (a.y-ERRORSIZE>bottom && a.y+ERRORSIZE>=bottom));
}

void CDrawError::Paint(Context &dc,paint_options options)
{
  dc.SelectBrush();

  switch (options)
  {
  case draw_selected:
	  dc.SelectPen(PS_SOLID,5,cSELECT);
	  break;
  case draw_selectable:
	  dc.SelectPen(PS_SOLID,5,cBUS_CLK);
	  break;
  default:
	  dc.SelectPen(PS_SOLID,5,cBUS);
  }

  dc.SetROP2(R2_COPYPEN);

  dc.Ellipse(CRect(a.x+ERRORSIZE,a.y+ERRORSIZE,a.x-ERRORSIZE,a.y-ERRORSIZE));
}


void CDrawError::BeginEdit(BOOL re_edit)
{
	theERCListBox.SetSelect(ErrorNumber);
}

void CDrawError::EndEdit()
{
}

BOOL CDrawError::CanEdit()
{
  return TRUE;
}



// Store the error in the design
CDrawingObject* CDrawError::Store()
{
  CDrawError *NewObject;

  NewObject = new CDrawError(m_pDesign);
  *NewObject = *this;

  m_pDesign->Add(NewObject);
  
  m_segment=1;

  m_pDesign->SetRepeat(NULL);

  return NewObject;
}



////// The tag object type //////

// Save the object to a file
void CDrawTag::Save(CStream& archive )
{
  WritePoint(archive,a);
  archive << TagName;
}


// Load the object to a file
void CDrawTag::Load(CStream& archive )
{
  	a = b = ReadPoint(archive);
  	archive >> TagName;
}






