/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



// This handles the actual drawing of objects
#include "stdafx.h"
#include "TinyCadView.h"
#include "diag.h"
#include "colour.h"
#include "option.h"
#include "LineUtils.h"
#include <math.h>


////// The Junction Class ////



CDrawJunction::CDrawJunction(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign )
{
	a=b=CPoint(0,0);
	m_segment=1;
	m_Undo_Level=0;
	m_Delete_Level=-1;
	is_stuck = FALSE;
}


void CDrawJunction::Load(CStream &archive)
{
  a = ReadPoint(archive);
  b=a;
  m_segment=0;
}


void CDrawJunction::Save(CStream &archive)
{
  WritePoint(archive,a);
}


BOOL CDrawJunction::IsInside(int left,int right,int top,int bottom)
{
  return (a.x>=left && a.x<=right && a.y>=top && a.y<=bottom);
}


ObjType CDrawJunction::GetType()
{
  return xJunction;
}


CString CDrawJunction::GetName() const
{
  return "Junction";
}

void CDrawJunction::Move(CPoint p)
{
  Display();

  BOOL is_junction;
  p = m_pDesign->GetStickyPoint(p,FALSE,TRUE,is_stuck,is_junction );

  a=p;
  b=a;
  m_segment=0;
  Display();
}

void CDrawJunction::LButtonDown(CPoint p, CPoint)
{
  Display();
  BOOL is_junction;
  p = m_pDesign->GetStickyPoint(p,FALSE,TRUE,is_stuck,is_junction );
  a=p;
  b=p;
  Store();
  Display();	// Write to screen
}

void CDrawJunction::Display( BOOL erase )
{
	// Invalidate the symbol
	CRect r( a.x,a.y,b.x,b.y);
	m_pDesign->InvalidateRect( r, erase, JUNCTION_SIZE + 1 );

   if (is_stuck)
   {
	 r.left = a.x - HIGHLIGHT_SIZE;
	 r.right = a.x + HIGHLIGHT_SIZE;
	 r.top = a.y - HIGHLIGHT_SIZE;
	 r.bottom = a.y + HIGHLIGHT_SIZE;
	 m_pDesign->InvalidateRect( r, erase, 3 );
   } 

}


void CDrawJunction::Paint(Context &dc,paint_options options)
{
  int js=JUNCTION_SIZE;
  CPoint br,tl;
  br=CPoint(a.x+js+1,a.y+js+1);
  tl=CPoint(a.x-js,a.y-js);

	dc.SetROP2(R2_COPYPEN);


  switch (options)
  {
  case draw_selected:
	  dc.SelectPen(PS_SOLID,1,cSELECT);
      dc.SelectBrush(cSELECT);
	  break;
  case draw_selectable:
	  dc.SelectPen(PS_SOLID,1,cJUNCTION_CLK);
      dc.SelectBrush(cJUNCTION_CLK);
	  break;
  default:
		dc.SelectPen(PS_SOLID,1,cJUNCTION);
		dc.SelectBrush(cJUNCTION);
  }

  dc.Ellipse(CRect(tl.x,tl.y,br.x,br.y));

  if (is_stuck)
  {
	  // Draw a nice circle to show the stickness...
	  dc.SelectBrush();
	  dc.SelectPen(PS_SOLID,5,cBUS);
	  dc.SetROP2(R2_COPYPEN);
	  dc.Ellipse(CRect(b.x+HIGHLIGHT_SIZE,b.y+HIGHLIGHT_SIZE,b.x-HIGHLIGHT_SIZE,b.y-HIGHLIGHT_SIZE));

  }
}


// Store the junction in the drawing
CDrawingObject* CDrawJunction::Store()
{
  CDrawJunction *NewObject;

  is_stuck = FALSE;
  NewObject = new CDrawJunction(m_pDesign);
  *NewObject = *this;

  m_pDesign->Add(NewObject);

  m_pDesign->SetRepeat(NULL);

  return NewObject;
}

