/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/




// This handles the actual drawing of objects

#include "stdafx.h"
#include "TinyCadView.h"
#include "diag.h"
#include "colour.h"
#include "option.h"


// The editing dialogs
extern CDlgERCListBox theERCListBox;

////////// The default actions for each object ///////////


CDrawingObject::CDrawingObject( CTinyCadDoc *pDesign )
{
	m_pDesign = pDesign;
	a = CPoint(0,0);
	b = CPoint(0,0);
	m_Delete_Level = -1;
	m_Undo_Level = -1;
	m_segment = 0;
}

BOOL CDrawingObject::IsConstruction()
{
	return FALSE;
}


void CDrawingObject::LButtonUp(CPoint)
{

}

// Is a point pointing at one of our fields?
int CDrawingObject::IsInsideField(CPoint)
{
	return -1;
}

void CDrawingObject::MoveField(int w, CPoint r)
{
	// Do nothing
}


// Set the new layer when the options change
void CDrawingObject::NewOptions()
{
}

void CDrawingObject::TagResources()
{

}



// Read the CPoints from a file (for Win16 compatibility)
CPoint CDrawingObject::ReadPoint(CStream &theFile)
{   
	CPoint r;

	theFile >> r;
	
	return r;
}


void CDrawingObject::WritePoint(CStream &theFile, CPoint a)
{
	theFile << a;
}


// The user has double clicked with the left hand button
void CDrawingObject::DblLButtonDown(CPoint,CPoint)
{
}


// Place the object into the design
void CDrawingObject::LButtonDown(CPoint p, CPoint)
{
  if (m_segment) {
	a=p;
	b=p;
	m_segment=!m_segment;
	Display();
  } else {
	b=p;
	Display();
	Store();
	a=b;
  }
}

// Find a string in this object
CString CDrawingObject::Find(const char *)
{
	return "";
}


// Move the object about the design
void CDrawingObject::Move(CPoint p)
{
  if (!m_segment) {
	Display();
	b=p;
	Display();
  }
}


// Cancel editing this part of the object in the design
BOOL CDrawingObject::RButtonDown(CPoint p, CPoint s)
{
  // If necessary remove the line from the screen
  Display();

  // Signal this is the end of this object
  return FALSE;
}

void CDrawingObject::RButtonUp( CPoint, CPoint )
{

}


void CDrawingObject::BeginEdit(BOOL re_edit)
{
}

void CDrawingObject::EndEdit()
{
  Display();
}

BOOL CDrawingObject::CanEdit()
{
  return FALSE;
}

BOOL CDrawingObject::DoesExist()
{
  return m_Undo_Level<=m_pDesign->GetUndoLevel() && (m_Delete_Level==-1 || m_Delete_Level>m_pDesign->GetUndoLevel() );
}

ObjType CDrawingObject::GetType()
{
  return xAnotation;
}


CString CDrawingObject::GetName() const
{
  return "??";
}

void CDrawingObject::Display( BOOL erase )
{
	if (a != b)
	{
		CRect r( a.x,a.y,b.x,b.y);
		m_pDesign->InvalidateRect( r, erase, 0 );
	}
}

void CDrawingObject::Paint( Context &dc, paint_options options)
{
}

void CDrawingObject::PaintHandles(Context & )
{
}

CDrawingObject* CDrawingObject::Store()
{
  m_pDesign->SetRepeat(NULL);

  return NULL;
}

double CDrawingObject::DistanceFromPoint( CPoint p )
{
	return IsInside( p.x,p.x,p.y,p.y ) ? 0 : 100.0;
}

BOOL CDrawingObject::IsInside(int left,int right,int top,int bottom)
{
  return !( (a.x<left && b.x<=left) || (a.x>right && b.x>=right)
        || (a.y<top && b.y<=top) || (a.y>bottom && b.y>=bottom));
}


// Save the object to a file
void CDrawingObject::Save(CStream& archive )
{
  WritePoint(archive,a);
  WritePoint(archive,b);
}


// Load the object to a file
void CDrawingObject::Load(CStream& archive )
{
 	a = ReadPoint(archive);
   	b = ReadPoint(archive);
}




// Rotate this object about a point
void CDrawingObject::Rotate(CPoint p,int dir)
{
  // Translate this point so the rotational point is the origin
  a = CPoint(a.x-p.x,a.y-p.y);
  b = CPoint(b.x-p.x,b.y-p.y);

  // Perfrom the rotation
  switch (dir) {
	case 2: // Left
		a = CPoint(a.y,-a.x);
		b = CPoint(b.y,-b.x);
		break;		
	case 3: // Right
		a = CPoint(-a.y,a.x);
		b = CPoint(-b.y,b.x);
		break;
	case 4: // Mirror
		a = CPoint(-a.x,a.y);
		b = CPoint(-b.x,b.y);
		break;
  }

  // Re-translate the points back to the original location
  a = CPoint(a.x+p.x,a.y+p.y);
  b = CPoint(b.x+p.x,b.y+p.y);
  
}

int CDrawingObject::SetCursor( CPoint p )
{
	return -1;
}

int CDrawingObject::SetCursorEdit( CPoint p )
{
	BOOL r = IsInside(p.x,p.x,p.y,p.y);
	return r ? 11 : -1;
}

void CDrawingObject::ContextMenu( CPoint p, UINT id )
{
}


// For the context menu
void CDrawingObject::FinishDrawing( CPoint p )
{
}

int  CDrawingObject::GetContextMenu()
{
	return IDR_EDITTOOL;
}


