// JunctionUtils.cpp: implementation of the CJunctionUtils class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "tinycad.h"
#include "TinyCadDoc.h"
#include "JunctionUtils.h"
#include "LineUtils.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CJunctionUtils::CJunctionUtils(CTinyCadDoc*	pDesign)
{
	m_pDesign = pDesign;
}

CJunctionUtils::~CJunctionUtils()
{

}


// Required for the checkCollection below...
static inline bool operator<( const CPoint &a, const CPoint &b )
{
	return CJunctionUtils::todo_point::comparePoint( a,b );
}


void CJunctionUtils::CheckJunctionRequirement( CPoint a, CPoint b )
{
  CLineUtils l(a,b);

  	// Is this enabled?
	if (!m_pDesign->GetOptions()->GetAutoJunc())
	{
		return;
	}

	typedef std::set<CPoint> checkCollection;
	checkCollection check;


    // Search for methods, and look at their pins
	drawingIterator it = m_pDesign->GetDrawingBegin();
	while (it != m_pDesign->GetDrawingEnd()) 
	{
		CDrawingObject *ObjPtr = *it;

		CPoint p;
	    CPoint r;

		switch (ObjPtr->GetType()) 
		{
			case xJunction:
				p = ObjPtr->a;
				
				if (l.DistanceFromPoint(p, r) <= CLineUtils::pointOnLineDistance)
				{
					check.insert( ObjPtr->a );
				}
				break;
			case xPower:
				p = ObjPtr->a;
				
				if (l.DistanceFromPoint(p, r) <= CLineUtils::pointOnLineDistance)
				{
					check.insert( ObjPtr->a );
				}
				break;
			case xWire:
				if (l.DistanceFromPoint(ObjPtr->a, r) <= CLineUtils::pointOnLineDistance)
				{
					check.insert( ObjPtr->a );
				}
				if (l.DistanceFromPoint(ObjPtr->b, r) <= CLineUtils::pointOnLineDistance)
				{
					check.insert( ObjPtr->b );
				}
				break;
			case xMethodEx3:
				{
					CRect s( ObjPtr->a.x, ObjPtr->a.y, ObjPtr->b.x, ObjPtr->b.y );
					s.NormalizeRect();
					if (l.IsInside(s.left- CLineUtils::pointOnLineDistance, s.right + CLineUtils::pointOnLineDistance, 
						s.top - CLineUtils::pointOnLineDistance, s.bottom + CLineUtils::pointOnLineDistance ))
					{
						CPoint tr;
						CDrawMethod *theMethod = static_cast<CDrawMethod*>(ObjPtr);
						int hold = m_pDesign->GetPart();
						m_pDesign->SetPart(theMethod->GetSubPart());

						drawingCollection method;
						theMethod->ExtractSymbol( tr, method );
						
						// Search the symbol for pins
						drawingIterator it = method.begin();
						while (it != method.end()) 
						{
							CDrawingObject *pointer = *it;
							CDrawPin *thePin = static_cast<CDrawPin*>(pointer);
							if (pointer->GetType()==xPinEx && !(thePin->IsPower()) && !(thePin->IsInvisible()))
							{
								// This is a valid pin...
								CPoint d = thePin->GetPosition(theMethod);
								if (l.DistanceFromPoint(d, r) <= CLineUtils::pointOnLineDistance)
								{
									check.insert( d );
								}
							}
							
							++ it;
						}
						m_pDesign->SetPart(hold);
					}
				}
			break;

		}

		++ it;
	}

	// Now check all of the locations...
	checkCollection::iterator itx = check.begin();
	while (itx != check.end())
	{
		CheckJunctionRequirement( *itx );
		++ itx;
	}

}

void CJunctionUtils::CheckJunctionRequirement( CPoint q )
{
  CPoint r(0,0);
  int items = 0;
  BOOL line_end = FALSE;

  	// Is this enabled?
	if (!m_pDesign->GetOptions()->GetAutoJunc())
	{
		return;
	}


    // Search for methods, and look at their pins
	drawingIterator it = m_pDesign->GetDrawingBegin();
	drawingIterator junc;
	bool have_junc = false;
	while (it != m_pDesign->GetDrawingEnd()) 
	{
		CDrawingObject *ObjPtr = *it;

		int hold;
		CPoint tr;

		switch (ObjPtr->GetType()) 
		{
			case xWire:
				#define theLine ((CDrawLine*)ObjPtr)
				{
					CPoint d;
					CLineUtils l( theLine->a, theLine->b );
					double distance = l.DistanceFromPoint( q, d );

					if (distance <= CLineUtils::pointOnLineDistance)
					{
						items ++;

						// A point mid-way along a line is worth 2 lines
						// (because we may have to break them in the middle)
						if (q != theLine->a && q != theLine->b)
						{
							items ++;
						}
						else
						{
							line_end = TRUE;
						}
					}
				}
			break;
			case xMethodEx3:
				{
					CRect s( ObjPtr->a.x, ObjPtr->a.y, ObjPtr->b.x, ObjPtr->b.y );
					s.NormalizeRect();
					s.left -= CLineUtils::pointOnLineDistance;
					s.right += CLineUtils::pointOnLineDistance;
					s.bottom += CLineUtils::pointOnLineDistance;
					s.top -= CLineUtils::pointOnLineDistance;

					if (s.PtInRect(q))
					{
						CDrawMethod *theMethod = static_cast<CDrawMethod*>(ObjPtr);
						hold = m_pDesign->GetPart();
						m_pDesign->SetPart(theMethod->GetSubPart());

						drawingCollection method;
						theMethod->ExtractSymbol( tr, method );
						
						// Search the symbol for pins
						drawingIterator it = method.begin();
						while (it != method.end()) 
						{
							CDrawingObject *pointer = *it;
							CDrawPin *thePin = static_cast<CDrawPin*>(pointer);
							if (pointer->GetType()==xPinEx && !(thePin->IsPower()) && !(thePin->IsInvisible()))
							{
								// This is a valid pin...
								CPoint d = thePin->GetPosition(theMethod);

								if (d == q)
								{
									items ++;
									line_end = TRUE;
								}
							}
							
							++ it;
						}
						m_pDesign->SetPart(hold);
					}
				}
			break;
			case xPower:
				{
					// This is a valid pin...
					CPoint d = ObjPtr->a;
					if (d == q)
					{
						items ++;
						line_end = TRUE;
					}
				}
				break;
			case xJunction:
				{
					// This is a valid pin...
					if (ObjPtr->a == q)
					{
						junc = it;
						have_junc = true;
					}
				}
				break;
		}

		++ it;
	}


  if ((items <= 2 || !line_end) && have_junc)
  {
	  // No junction required, delete it!
	  CDrawingObject *pJunction = *junc;
	  m_pDesign->Erase( junc );
	  delete pJunction;
  }
  else if (items > 2 && !have_junc && line_end)
  {
	  m_pDesign->AddNewJunction( q );
  }
}


// Add an object to the todo list
void CJunctionUtils::AddObjectToTodo( CDrawingObject *obj )
{
	// Is this enabled?
	if (!m_pDesign->GetOptions()->GetAutoJunc())
	{
		return;
	}

	// Extract the todo list...
	switch (obj->GetType()) 
	{
		case xWire:
			m_todo.insert( todo_point( obj->a, obj->b ) );
			break;
		case xMethodEx3:
			{
				CPoint tr;
				CDrawMethod *theMethod = static_cast<CDrawMethod*>(obj);
				int hold = m_pDesign->GetPart();
				m_pDesign->SetPart(theMethod->GetSubPart());

				drawingCollection method;
				theMethod->ExtractSymbol( tr, method );
				
				// Search the symbol for pins
				drawingIterator it = method.begin();
				while (it != method.end()) 
				{
					CDrawingObject *pointer = *it;
					CDrawPin *thePin = static_cast<CDrawPin*>(pointer);
					if (pointer->GetType()==xPinEx && !(thePin->IsPower()) && !(thePin->IsInvisible()))
					{
						// This is a valid pin...
						CPoint d = thePin->GetPosition(theMethod);
						m_todo.insert( todo_point( d ) );
					}
					
					++ it;
				}
				m_pDesign->SetPart(hold);
			}
		break;
		case xPower:
			m_todo.insert( todo_point( obj->a ) );
			break;
		case xJunction:
			m_todo.insert( todo_point( obj->a ) );
			break;
		}


}

void CJunctionUtils::CheckTodoList()
{
	// Is this enabled?
	if (!m_pDesign->GetOptions()->GetAutoJunc())
	{
		return;
	}

	// Now check-up on the junctions
	todoList::iterator it = m_todo.begin();
	while (it != m_todo.end())
	{
	  if ((*it).a == (*it).b)
	  {
		  CheckJunctionRequirement( (*it).a );
	  }
	  else
	  {
		  CheckJunctionRequirement( (*it).a, (*it).b );
	  }
	  ++ it;
	}

	m_todo.erase( m_todo.begin(), m_todo.end() );
}
