/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __LIBRARYSTORE_H__
#define __LIBRARYSTORE_H__

#include "Symbol.h"

class symbolCompare
{
public:
	bool operator()( const CString &a, const CString &b ) const
	{
		return stricmp(a,b) < 0;
	}
};

class CLibraryStore
{
public:
	typedef std::map<CString, CLibraryStoreSymbol,symbolCompare>	symbolCollection;
	symbolCollection	Symbols;	// The collection of symbols

	CString			m_name;					// The name of this library

	CLibraryStore();						// The constructor
	virtual ~CLibraryStore();				// The destructor


	// Get a symbol from this library
	CLibraryStoreSymbol *Extract(const char *);	

	// Find a string in the symbol descriptions
	virtual void Find(const char *,CListBox *);

	// Check to see if the item exists in this library
	virtual BOOL DoesExist( CLibraryStoreSymbol *pSymbol );

	// Re-read this library
	virtual void ReRead();


	// Get the Archive to load from
	virtual CStream *GetMethodArchive( CLibraryStoreSymbol *symbol ) = 0;

	// Attach this library to a file
	virtual void Attach(const char *filename) = 0;
	
	// Upgrade to the latest version of the library system
	virtual BOOL Upgrade(CLibraryStore *) = 0;


	// Write a symbol to this library
	virtual void Store(CLibraryStoreSymbol *nwSymbol, CTinyCadSymbolDoc &document ) = 0;

	// Write back a symbol collection and rebuild methods file
	// (i.e. partial tidy)
	virtual void SaveSymbolCollection( symbolCollection &temp_symbols ) = 0;

	// Do any idle time tasks...
	virtual void OnIdle() = 0;

	// Create a new library database
	virtual bool Create( const char *filename ) = 0;
};

typedef std::list<CLibraryStore*>	libraryCollection;
typedef libraryCollection::iterator libraryIterator;


#endif // __LIBRARYSTORE_H__
