/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// OptionsSheets.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "OptionsSheets.h"
#include "OptionsPropertySheet.h"
#include "TinyCadDoc.h"
#include "Revision.h"
#include "Registry.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(COptionsGrid, CPropertyPage)
IMPLEMENT_DYNCREATE(COptionsAutosnap, CPropertyPage)


/////////////////////////////////////////////////////////////////////////////
// COptionsGrid property page

COptionsGrid::COptionsGrid() : CPropertyPage(COptionsGrid::IDD)
{
	//{{AFX_DATA_INIT(COptionsGrid)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

COptionsGrid::~COptionsGrid()
{
}

void COptionsGrid::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COptionsGrid)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(COptionsGrid, CPropertyPage)
	//{{AFX_MSG_MAP(COptionsGrid)
	ON_BN_CLICKED(GRIDBOX_MM, OnMm)
	ON_BN_CLICKED(GRIDBOX_INCHES, OnInches)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


CTinyCadDoc *COptionsGrid::GetDocument()
{
	return static_cast<COptionsPropertySheet*>(GetParent())->m_pDocument;
}

BOOL COptionsGrid::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	// Get the document...
	CTinyCadDoc *pDocument = GetDocument();
	Option *pOptions = pDocument->GetOptions();
	TransformSnap *pSnap = &pDocument->m_snap;

	Grid = pSnap->GetAccurateGrid();
	ShowGrid = pOptions->ShowGrid();
	SnapGrid = pSnap->GetGridSnap();
	Units = pOptions->GetUnits();
	ScrollBar = pOptions->GetScrollBar();

	CheckDlgButton(GRIDBOX_SHOW,ShowGrid);
	CheckDlgButton(GRIDBOX_SNAP,SnapGrid);

	SetDlgItemInt(GRIDBOX_SCROLLBAR,ScrollBar);

	CheckRadioButton(GRIDBOX_MM,GRIDBOX_INCHES,GRIDBOX_MM+Units);
	if (Grid == NormalGrid)
		CheckRadioButton(GRIDBOX_NORMAL,GRIDBOX_USER,GRIDBOX_NORMAL);
	else if (Grid == FineGrid)
		CheckRadioButton(GRIDBOX_NORMAL,GRIDBOX_USER,GRIDBOX_FINE);
	else
		CheckRadioButton(GRIDBOX_NORMAL,GRIDBOX_USER,GRIDBOX_USER);

	// Set up the display of the grid spacing correctly
	OnChange();	


	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void COptionsGrid::OnMm() 
{
	OnChange();
}

void COptionsGrid::OnInches() 
{
	OnChange();
}

void COptionsGrid::OnChange()
{
	// The current grid in .1mm's
	LONG ScaledGrid;

	if (GetCheckedRadioButton(GRIDBOX_MM,GRIDBOX_INCHES) == GRIDBOX_MM) 
	{
		ScaledGrid = Grid / PIXELSPERMM;
		SetDlgItemText(GRIDBOX_UNITS,".1mm");
		Units = 0;
	}
	else 
	{
		SetDlgItemText(GRIDBOX_UNITS,".001\"");
		ScaledGrid = (1000*Grid) / (254 * PIXELSPERMM);
		Units = 1;
	}

	SetDlgItemInt(GRIDBOX_SPACING,ScaledGrid);
}


BOOL COptionsGrid::OnApply() 
{
	ShowGrid  = IsDlgButtonChecked(GRIDBOX_SHOW);
	SnapGrid  = IsDlgButtonChecked(GRIDBOX_SNAP);

	int hold = GetDlgItemInt(GRIDBOX_SCROLLBAR);
	if (hold > 0)
		ScrollBar = hold;

	hold = GetCheckedRadioButton(GRIDBOX_MM,GRIDBOX_INCHES);
	if (hold == GRIDBOX_MM)
		Units = 0;
	else
		Units = 1;


	hold = GetCheckedRadioButton(GRIDBOX_NORMAL,GRIDBOX_USER);
	switch (hold) {
		case GRIDBOX_NORMAL:
			Grid = NormalGrid;
			break;
		case GRIDBOX_FINE:
			Grid = FineGrid;
			break;
		case GRIDBOX_USER:
			Grid = GetDlgItemInt(GRIDBOX_SPACING);
			if (Units == 0)
				Grid = (Grid * PIXELSPERMM);
			else
				Grid = (Grid * 254 * PIXELSPERMM) / 1000;
			break;
	}

	if (Grid <=0 )
		Grid = FineGrid;

	// Write these settings back...
	CTinyCadDoc *pDocument = GetDocument();
	Option *pOptions = pDocument->GetOptions();
	TransformSnap *pSnap = &pDocument->m_snap;

	pSnap->SetAccurateGrid(Grid);
	pOptions->SetGridShow( ShowGrid );
	pSnap->SetGridSnap( SnapGrid );
	pOptions->SetUnits( Units );
	pOptions->SetScrollBar( ScrollBar );


	// Write the settings to the registry
	Registry theRegistry;
	WORD w_hold;

	w_hold = ShowGrid;
	theRegistry.Place("ShowGrid",w_hold);

	w_hold = Units;
	theRegistry.Place("Units",w_hold);

	w_hold = ScrollBar;
	theRegistry.Place("ScrollBar",w_hold);

	w_hold = Grid;
	theRegistry.Place("Grid",w_hold);

	
	return CPropertyPage::OnApply();
}











/////////////////////////////////////////////////////////////////////////////
// COptionsAutosnap property page

COptionsAutosnap::COptionsAutosnap() : CPropertyPage(COptionsAutosnap::IDD)
{
	//{{AFX_DATA_INIT(COptionsAutosnap)
	m_Autodrag_Enable = FALSE;
	m_Autosnap_Enable = FALSE;
	m_Autosnap_Range = 0;
	m_Units = _T("");
	m_Autojunc_Enable = FALSE;
	//}}AFX_DATA_INIT
}

COptionsAutosnap::~COptionsAutosnap()
{
}

void COptionsAutosnap::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COptionsAutosnap)
	DDX_Control(pDX, IDC_AUTOSNAP_RANGE, m_Range_Ctrl);
	DDX_Check(pDX, IDC_AUTODRAG_ENABLE, m_Autodrag_Enable);
	DDX_Check(pDX, IDC_AUTOSNAP_ENABLE, m_Autosnap_Enable);
	DDX_Text(pDX, IDC_AUTOSNAP_RANGE, m_Autosnap_Range);
	DDX_Text(pDX, IDC_AUTOSNAP_UNITS, m_Units);
	DDX_Check(pDX, IDC_AUTOJUNC_ENABLE, m_Autojunc_Enable);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(COptionsAutosnap, CPropertyPage)
	//{{AFX_MSG_MAP(COptionsAutosnap)
	ON_BN_CLICKED(IDC_AUTOSNAP_ENABLE, OnAutosnapEnable)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



CTinyCadDoc *COptionsAutosnap::GetDocument()
{
	return static_cast<COptionsPropertySheet*>(GetParent())->m_pDocument;
}


BOOL COptionsAutosnap::OnInitDialog() 
{
	// Read the active state from the other dialogue
	// Create the setup
	Units = static_cast<COptionsPropertySheet*>(GetParent())->m_Page1.Units;

	CTinyCadDoc *pDocument = GetDocument();
	Option *pOptions = pDocument->GetOptions();

	m_Autodrag_Enable = pOptions->GetAutoDrag();
	m_Autosnap_Enable = pOptions->GetAutoSnap();
	m_Autojunc_Enable = pOptions->GetAutoJunc();
	m_Autosnap_Range = Units == 0 ? 
		pOptions->GetAutoSnapRange() * 10 / PIXELSPERMM : 
		(10000*pOptions->GetAutoSnapRange()) / (254 * PIXELSPERMM);

	m_Units = (Units == 0) ? ".1mm" : ".001\"";

	// Now create the document
	CPropertyPage::OnInitDialog();
	
	m_Range_Ctrl.EnableWindow( m_Autosnap_Enable );
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void COptionsAutosnap::OnAutosnapEnable() 
{
	// Enable disable/the range dialogue
	UpdateData( TRUE );
	m_Range_Ctrl.EnableWindow( m_Autosnap_Enable );
	
}

BOOL COptionsAutosnap::OnApply() 
{
	UpdateData( TRUE );

	CTinyCadDoc *pDocument = GetDocument();
	Option *pOptions = pDocument->GetOptions();
	int range;

	if (Units == 0)
		range = (m_Autosnap_Range * PIXELSPERMM) / 10;
	else
		range = (m_Autosnap_Range * 254 * PIXELSPERMM) / 10000;


	pOptions->SetAutoDrag( m_Autodrag_Enable );
	pOptions->SetAutoSnap( m_Autosnap_Enable );
	pOptions->SetAutoJunc( m_Autojunc_Enable );
	pOptions->SetAutoSnapRange( range );

	// Write the settings to the registry
	Registry theRegistry;
	WORD w_hold;

	w_hold = m_Autodrag_Enable;
	theRegistry.Place("AutoDrag",w_hold);

	w_hold = m_Autosnap_Enable;
	theRegistry.Place("AutoSnap",w_hold);

	w_hold = m_Autojunc_Enable;
	theRegistry.Place("AutoJunc",w_hold);

	w_hold = range;
	theRegistry.Place("AutoDragRange",w_hold);

	return CPropertyPage::OnApply();
}

BOOL COptionsAutosnap::OnSetActive() 
{
	// Read the active state from the other dialogue
	// Create the setup
	int new_Units = static_cast<COptionsPropertySheet*>(GetParent())->m_Page1.Units;

	if (new_Units != Units)
	{
		// Get the old scaling...
		UpdateData( TRUE );
		int range;
		if (Units == 0)
			range = (m_Autosnap_Range * PIXELSPERMM) / 10;
		else
			range = (m_Autosnap_Range * 254 * PIXELSPERMM) / 10000;


		Units = new_Units;
		CTinyCadDoc *pDocument = GetDocument();
		Option *pOptions = pDocument->GetOptions();
		m_Autosnap_Range = Units == 0 ? 
			range * 10 / PIXELSPERMM : 
			(10000*range) / (254 * PIXELSPERMM);

		m_Units = (Units == 0) ? ".1mm" : ".001\"";
		UpdateData( FALSE );
	}

	return CPropertyPage::OnSetActive();
}
