/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "TinyCadView.h"
#include "registry.h"
#include "colour.h"
#include "TinyCad.h"
#include "LibraryDb.h"
#include "DlgLibraryBox.h"



////// The library box dialog //////

#define FIELD_LENGTH	35

CDlgLibraryBox::CDlgLibraryBox(CWnd *pParentWnd) 
	: CDialog( IDD_LIBRARY, pParentWnd ) 
{ 
	//{{AFX_DATA_INIT(CDlgLibraryBox)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CDlgLibraryBox::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgLibraryBox)
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP( CDlgLibraryBox, CDialog )
	//{{AFX_MSG_MAP(CDlgLibraryBox)
	ON_COMMAND( LIBRARYBOX_ADD, OnAdd )
	ON_COMMAND( LIBRARYBOX_REMOVE, OnRemove )
	ON_COMMAND( LIBRARYBOX_EDIT, OnEdit )	
	ON_LBN_DBLCLK(LIBRARYBOX_AVAILABLE, OnDblclkAvailable)
	ON_BN_CLICKED(LIBRARYBOX_NEW, OnNew)
	ON_BN_CLICKED(LIBRARYBOX_UPGRADE, OnUpgrade)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CDlgLibraryBox::OnInitDialog()
{

  // Set up the list box with the current library names in it
  theListBox = (CListBox *)GetDlgItem(LIBRARYBOX_AVAILABLE);
  theListBox->ResetContent();

  libraryIterator lib_it = static_cast<CTinyCadApp*>(AfxGetApp())->GetLibraryBegin();
  while (lib_it != static_cast<CTinyCadApp*>(AfxGetApp())->GetLibraryEnd()) 
  {
	theListBox->AddString((*lib_it)->m_name);
	++ lib_it;
  }

  return TRUE;
}


void CDlgLibraryBox::OnOK()
{

  // Write the new library details back to the registry
  Registry theRegistry;
  CString theString = "";
  libraryIterator lib_it = static_cast<CTinyCadApp*>(AfxGetApp())->GetLibraryBegin();
  while (lib_it != static_cast<CTinyCadApp*>(AfxGetApp())->GetLibraryEnd()) 
  {
	  if (!theString.IsEmpty())
	  {
		  theString += ",";
	  }

	theString += (*lib_it)->m_name;
	++ lib_it;
  }
  theRegistry.Place("Libraries",theString);

  EndDialog( IDOK );
}


void CDlgLibraryBox::OnUpgrade() 
{
  CLibraryStore *lib=LibSelected();

  if (lib==NULL)
	return;

	if (static_cast<CTinyCadApp*>(AfxGetApp())->IsLibInUse( lib ))
	{
		// We cannot upgrade whilst in use
		AfxMessageBox( IDS_NOUPGRADE );
		return;
	}

    CLibraryStore *NewLib = new CLibraryDb;
	if (!lib->Upgrade( NewLib ))
	{
		// Upgrade failed...
		delete NewLib;
	}
	else
	{
		// First remove the old library from this system
		static_cast<CTinyCadApp*>(AfxGetApp())->RemoveLibrary( lib );
	    delete lib;

		// Now add the new library...
	    static_cast<CTinyCadApp*>(AfxGetApp())->AddLibrary( NewLib );
	}

   // Finally update the symbol picker...
  static_cast<CTinyCadApp*>(AfxGetApp())->ResetAllSymbols();

}


// Select a library for editing
void CDlgLibraryBox::OnEdit()
{
  CLibraryStore *lib=LibSelected();

  if (lib==NULL)
	return;

  static_cast<CTinyCadApp*>(AfxGetApp())->editLibrary( lib );

  EndDialog(IDOK);
}


// Remove a library from the list of libraries in use
void CDlgLibraryBox::OnRemove()
{
  int WhichItem;
  char Buffer[STRLEN];

  // Get the item which has been selected
  if ((WhichItem = theListBox->GetCurSel())==LB_ERR)
	return;

  // Now calculate which library this is
  theListBox->GetText(WhichItem,Buffer);

  libraryIterator lib_it = static_cast<CTinyCadApp*>(AfxGetApp())->GetLibraryBegin();
  while (lib_it != static_cast<CTinyCadApp*>(AfxGetApp())->GetLibraryEnd()) 
  {
    CLibraryStore *lib = *lib_it;
	if (strcmp(lib->m_name,Buffer)==0) 
	{
		static_cast<CTinyCadApp*>(AfxGetApp())->RemoveLibrary( lib );
	    delete lib;
		break;
	}

	++ lib_it;
  }


  // Now remove this from the list box in the dialog box
  theListBox->DeleteString(WhichItem);

   // Finally update the symbol picker...
  static_cast<CTinyCadApp*>(AfxGetApp())->ResetAllSymbols();

}


// Add a new library to this list box
void CDlgLibraryBox::OnAdd()
{
  OPENFILENAME ofn;
  char szFile[256], szFileTitle[256];

  strcpy(szFile, "*.mdb;*.idx\0");

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
  ofn.lpstrFilter = "Library file (*.mdb, *idx)\0*.mdb;*.idx\0All files (*.*)\0*.*\0\0";
  ofn.lpstrCustomFilter = NULL;
  ofn.nMaxCustFilter = 0;
  ofn.nFilterIndex = 1;
  ofn.lpstrFile = szFile;
  ofn.nMaxFile = sizeof(szFile);
  ofn.lpstrFileTitle = szFileTitle;
  ofn.nMaxFileTitle = sizeof(szFileTitle);
  ofn.lpstrInitialDir = NULL;
  ofn.lpstrTitle = NULL;
  ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
  ofn.nFileOffset = 0;
  ofn.nFileExtension = 0;
  ofn.lpstrDefExt = ".mdb";

  if (!GetOpenFileName(&ofn))
       	return;

  // Remove the extension  
  for (int lp=0;szFile[lp]!=0 && szFile[lp]!='.'; lp++) ;
  szFile[lp]=0;

  // Create the library
  CLibraryStore *NewLib;
  if (stricmp(szFile+lp+1,"idx") == 0)
  {
	NewLib = new CLibraryFile;
  }
  else
  {
	NewLib = new CLibraryDb;
  }
  NewLib->Attach(szFile);

  // Now add it to the library linked list
  static_cast<CTinyCadApp*>(AfxGetApp())->AddLibrary( NewLib );

  // .. and to the list box
  theListBox->AddString(NewLib->m_name);

  // Finally update the symbol picker...
  static_cast<CTinyCadApp*>(AfxGetApp())->ResetAllSymbols();

}


// Find out which of the libraries has been selected
CLibraryStore *CDlgLibraryBox::LibSelected()
{
  int WhichItem;
  char Buffer[STRLEN];
 
  // Get the item which has been selected
  if ((WhichItem = theListBox->GetCurSel())==LB_ERR)
	return NULL;

  // Now calculate which library this is
  theListBox->GetText(WhichItem,Buffer);

  libraryIterator lib_it = static_cast<CTinyCadApp*>(AfxGetApp())->GetLibraryBegin();
  while (lib_it != static_cast<CTinyCadApp*>(AfxGetApp())->GetLibraryEnd()) 
  {
    CLibraryStore *lib = *lib_it;
	if (strcmp(lib->m_name,Buffer)==0) 
	{
		return lib;
	}

	++ lib_it;
  }

  return NULL;
}


void CDlgLibraryBox::OnDblclkAvailable() 
{
	OnEdit();
}

void CDlgLibraryBox::OnNew() 
{
  OPENFILENAME ofn;
  char szFile[256], szFileTitle[256];

  strcpy(szFile, "*.mdb\0");

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
  ofn.lpstrFilter = "Library database (*.mdb)\0*.mdb\0Old Library File (*.idx)\0*.idx\0All files (*.*)\0*.*\0\0";
  ofn.lpstrCustomFilter = NULL;
  ofn.nMaxCustFilter = 0;
  ofn.nFilterIndex = 1;
  ofn.lpstrFile = szFile;
  ofn.nMaxFile = sizeof(szFile);
  ofn.lpstrFileTitle = szFileTitle;
  ofn.nMaxFileTitle = sizeof(szFileTitle);
  ofn.lpstrInitialDir = NULL;
  ofn.lpstrTitle = NULL;
  ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
  ofn.nFileOffset = 0;
  ofn.nFileExtension = 0;
  ofn.lpstrDefExt = "mdb";

  if (!GetSaveFileName(&ofn))
       	return;

  // Remove the extension  
  for (int lp=0;szFile[lp]!=0 && szFile[lp]!='.'; lp++) ;
  szFile[lp]=0;

  // Create the library object
  CLibraryDb *NewLib;
  NewLib = new CLibraryDb;

  // .. and create the file
  if (!NewLib->Create(szFile))
  {
	  delete NewLib;
	  return;
  }


  // Now add it to the library linked list
  static_cast<CTinyCadApp*>(AfxGetApp())->AddLibrary( NewLib );

  // .. and to the list box
  theListBox->AddString(NewLib->m_name);
}

